/***************************************************************************
 *   Copyright (C) 2007 by    *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "acsalarmesa1.h"
#include <qtimer.h>
#include <qsettings.h>

	char tmBuffer[80];
/*
  timestamp() 
  send a timestamp to standard output
*/
void timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}

acsalarmesa1::acsalarmesa1(QWidget *parent) : QWidget(parent)
{
	// setup User Interface
	try {
		ui.setupUi(this);
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot setup UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// connect to control access server
	try {
		plc_server = new Tango::DeviceProxy("f/access_control/safety");
		// plc_server = new Tango::DeviceProxy("ken/plc/pss");
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	refresh();
	// init timer
	try {
		timer = new QTimer(this);
		timer->setInterval(1000);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
}

acsalarmesa1::~acsalarmesa1()
{
}    
    
int acsalarmesa1::myReadBoolArray(const char *attrName, vector<bool> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acsalarmesa1::myReadAttribute(const char *attrName)
{
	try {
		myAttribute = plc_server->read_attribute(attrName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

void acsalarmesa1::refresh()
{
	vector<bool> stat;
	// set search hinibition causes
	if (myReadBoolArray("Undulator_search_signals_esa1", stat)) {
		ui.checkBoxHinibit_0->setCheckState(stat[0]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_1->setCheckState(stat[1]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_2->setCheckState(stat[2]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_3->setCheckState(stat[3]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_4->setCheckState(stat[4]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_5->setCheckState(stat[5]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_6->setCheckState(stat[6]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_7->setCheckState(stat[7]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_8->setCheckState(stat[8]? Qt::Checked: Qt::Unchecked);
		ui.checkBoxHinibit_9->setCheckState(stat[9]? Qt::Checked: Qt::Unchecked);
	}
	else {
		ui.checkBoxHinibit_0->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_1->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_2->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_3->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_4->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_5->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_6->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_7->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_8->setCheckState(Qt::PartiallyChecked);
		ui.checkBoxHinibit_9->setCheckState(Qt::PartiallyChecked);
	}
}
